"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.MapCacheManager = void 0;
const helper_1 = require("../helper");
const exception_1 = require("../exception");
class MapCacheManager {
    constructor(options) {
        this._entriesCount = 0;
        this._options = Object.assign({ bucket: {}, maxEntries: helper_1.Defaults.MaxObjectEntrySize }, (options !== null && options !== void 0 ? options : {}));
        this.get = this.get.bind(this);
        this.set = this.set.bind(this);
        this.has = this.has.bind(this);
        this.clear = this.clear.bind(this);
        this._resolve = this._resolve.bind(this);
    }
    static getInstance(options) {
        if (!MapCacheManager.instance)
            MapCacheManager.instance = new MapCacheManager(options);
        return MapCacheManager.instance;
    }
    usedSpace() {
        return this._entriesCount;
    }
    availableSpace() {
        return this._options.maxEntries - this._entriesCount;
    }
    calculateSpace(_, __) {
        return 1;
    }
    has(configOrKey) {
        return this._resolve(configOrKey).exists;
    }
    get(configOrKey) {
        const resolve = this._resolve(configOrKey);
        if (!resolve.exists)
            return undefined;
        const value = this._options.bucket[resolve.key];
        const parsed = JSON.parse(this._options.encryptor ? this._options.encryptor.decrypt(value) : value);
        return {
            value: parsed.value,
            date: new Date(parsed.date),
        };
    }
    getValue(configOrKey) {
        var _a;
        return ((_a = this.get(configOrKey)) !== null && _a !== void 0 ? _a : {}).value;
    }
    set(configOrKey, value) {
        if (this.calculateSpace(configOrKey, value) > this.availableSpace()) {
            throw new exception_1.NoSufficientCacheSpaceLeftError();
        }
        const resolve = this._resolve(configOrKey);
        const entryStr = helper_1.Utils.safeStringify({
            value,
            date: new Date(),
        });
        this._options.bucket[resolve.key] = (this._options.encryptor ? this._options.encryptor.encrypt(entryStr) : entryStr);
        if (!resolve.exists)
            this._entriesCount++;
    }
    remove(configOrKey) {
        const resolve = this._resolve(configOrKey);
        if (!resolve.exists)
            return;
        delete this._options.bucket[resolve.key];
        this._entriesCount--;
    }
    clear() {
        this._options.bucket = {};
        this._entriesCount = 0;
    }
    find(cond) {
        const found = [];
        for (const key in this._options.bucket) {
            if (cond(key))
                found.push(key);
        }
        return found;
    }
    _resolve(configOrKey) {
        let key = ((typeof configOrKey === "string") ? configOrKey : helper_1.Utils.buildCacheKey(configOrKey));
        key = (this._options.encryptKey && this._options.encryptor ? this._options.encryptor.encrypt(key) : key);
        const exists = key in this._options.bucket;
        return { key, exists };
    }
}
exports.MapCacheManager = MapCacheManager;
