"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = httpConnector;
const core_1 = require("../../core");
const node_classes_1 = __importDefault(require("../../helper/node_classes"));
const helper_1 = require("../../helper");
const types_1 = require("../../types");
// TODO treat proxy
// TODO treat cancellation
function httpConnector(config, queueRequest) {
    return new Promise((resolvePromise, rejectPromise) => {
        var _a, _b, _c, _d, _e, _f, _g, _h;
        let rejected = false;
        function reject(error) {
            rejected = true;
            rejectPromise(error);
        }
        function resolve(response) {
            var _a, _b, _c;
            if (rejected)
                return;
            if (response.__cached__) {
                (_a = config.interceptor) === null || _a === void 0 ? void 0 : _a.invoke(core_1.HandlerType.HTTP_POST_REQUEST, config);
                (_b = config.interceptor) === null || _b === void 0 ? void 0 : _b.invoke(core_1.HandlerType.HTTP_PRE_RESPONSE, config, response);
            }
            (_c = config.interceptor) === null || _c === void 0 ? void 0 : _c.invoke(core_1.HandlerType.HTTP_POST_RESPONSE, config, response);
            resolvePromise(response);
        }
        if (config.dynamicConfig)
            config = config.dynamicConfig(config);
        const preRequestCachedResults = (_a = config.interceptor) === null || _a === void 0 ? void 0 : _a.invoke(core_1.HandlerType.HTTP_PRE_REQUEST, config).filter((r) => (r === null || r === void 0 ? void 0 : r.__cached__) === true);
        if (preRequestCachedResults === null || preRequestCachedResults === void 0 ? void 0 : preRequestCachedResults.length) {
            resolve(preRequestCachedResults[0]);
            return;
        }
        const headers = (_b = config.headers) !== null && _b !== void 0 ? _b : {};
        let data = config.data;
        const headerNamesMap = Object.keys(headers).reduce((acc, header) => {
            acc[header.toLowerCase()] = header;
            return acc;
        }, {});
        if (!helper_1.Utils.envIsBrowser() && !('user-agent' in headerNamesMap)) {
            headers['User-Agent'] = `kyofuuc/${helper_1.Defaults.VERSION}`;
        }
        else {
            if (!headers[headerNamesMap['user-agent']]) {
                delete headers[headerNamesMap['user-agent']];
            }
        }
        if (data && !helper_1.Utils.isStream(data)) {
            if (!config.requestType) {
                if (helper_1.Utils.isString(data)) {
                    config.requestType = types_1.RequestType.TEXT;
                }
                else if (helper_1.Utils.isFormData(data)) {
                    config.requestType = types_1.RequestType.FORM_DATA;
                }
                else if (helper_1.Utils.isURLSearchParams(data)) {
                    config.requestType = types_1.RequestType.URL_SEARCH_PARAMS;
                }
                else if (helper_1.Utils.isPlainObject(data) || helper_1.Utils.isObject(data) || helper_1.Utils.isArray(data)) {
                    config.requestType = types_1.RequestType.JSON;
                }
                else if (helper_1.Utils.isArrayBuffer(data)) {
                    config.requestType = types_1.RequestType.ARRAY_BUFFER;
                }
                else if (Buffer.isBuffer(data)) {
                    config.requestType = types_1.RequestType.BUFFER;
                }
                else {
                    return reject(helper_1.Utils.kyofuucError('Request data after must be one of these ArrayBuffer, string, Stream, Form Data, Json Object, URLSearchParams, or Buffer', config, helper_1.ErrorCode.INVALID_REQUEST_TYPE));
                }
            }
            if (!(data = (0, types_1.transformRequestData)(config, data, !headerNamesMap['content-type'], headers, reject)))
                return;
            if (config.maxContentLength && data.length > config.maxContentLength) {
                return reject(helper_1.Utils.kyofuucError('Request body larger than max content length limit', config, helper_1.ErrorCode.REQUEST_EXCEEDS_MAXIMUM_LENGTH));
            }
            if (!headerNamesMap['content-length']) {
                headers['Content-Length'] = data.length;
            }
        }
        if (config.auth && headerNamesMap.authorization) {
            delete headers[headerNamesMap.authorization];
        }
        if (config.bearer && !headerNamesMap.authorization) {
            headers["Authorization"] = `Bearer ${config.bearer}`;
        }
        if (config.xsrf) {
            const xsrfValue = (_c = (config.xsrf.cookieName ? helper_1.Utils.getCookie((document !== null && document !== void 0 ? document : {}), config.xsrf.cookieName) : undefined)) !== null && _c !== void 0 ? _c : config.xsrf.value;
            headers[config.xsrf.headerName] = xsrfValue;
        }
        let isHttpsRequest = ((_d = config.parsed) === null || _d === void 0 ? void 0 : _d.protocol) === "https";
        let agent = isHttpsRequest ? config.httpsAgent : config.httpAgent;
        const options = {
            agent,
            headers,
            method: config.method,
            path: (_e = config.parsed) === null || _e === void 0 ? void 0 : _e.pathname,
            auth: helper_1.Utils.stringifyAuth(config.auth),
            maxBodyLength: config.maxContentLength,
            withCredentials: config.withCredentials,
            insecureHTTPParser: config.insecureHTTPParser,
            agents: { http: config.httpAgent, https: config.httpsAgent },
        };
        if (config.socketPath) {
            options.socketPath = config.socketPath;
        }
        else {
            options.port = (_f = config.parsed) === null || _f === void 0 ? void 0 : _f.port;
            options.hostname = (_g = config.parsed) === null || _g === void 0 ? void 0 : _g.hostname;
        }
        const redirectsResponses = [];
        const transport = ((_h = config.transport) !== null && _h !== void 0 ? _h : (isHttpsRequest ? node_classes_1.default.https : node_classes_1.default.http));
        function processResponse(res, lastReq, onSuccess, onError, finalRequest = false) {
            var _a, _b, _c;
            const response = {
                config,
                request: lastReq,
                __cached__: false,
                headers: res.headers,
                status: res.statusCode,
                statusText: res.statusMessage,
            };
            if (finalRequest) {
                (_a = config.interceptor) === null || _a === void 0 ? void 0 : _a.invoke(core_1.HandlerType.HTTP_PRE_RESPONSE, config, res);
            }
            if (!config.responseType) {
                const contentType = (_c = ((_b = response.headers) !== null && _b !== void 0 ? _b : {})['content-type']) === null || _c === void 0 ? void 0 : _c.toLowerCase();
                if (response.status === 204 || (contentType === null || contentType === void 0 ? void 0 : contentType.includes("nocontent"))) {
                    config.responseType = types_1.ResponseType.NOCONTENT;
                }
                else if (contentType === null || contentType === void 0 ? void 0 : contentType.includes("json")) {
                    config.responseType = types_1.ResponseType.JSON;
                }
                else if (contentType === null || contentType === void 0 ? void 0 : contentType.includes("octet-stream")) {
                    config.responseType = types_1.ResponseType.STREAM;
                }
                else {
                    config.responseType = types_1.ResponseType.TEXT;
                }
            }
            if (config.responseType === types_1.ResponseType.STREAM) {
                response.body = res;
                (0, types_1.transformResponseData)(config, response, res, onSuccess, onError);
                return;
            }
            let totalResponseBytes = 0;
            let responseBuffer = [];
            res.on('data', (chunk) => {
                responseBuffer.push(chunk);
                totalResponseBytes += chunk.length;
                if (config.maxContentLength && totalResponseBytes > config.maxContentLength) {
                    // stream.destoy() emit aborted event before calling reject() on Node.js v16
                    rejected = true;
                    res.destroy();
                    onError(helper_1.Utils.kyofuucError('Max content length of ' + config.maxContentLength + ' exceeded', config, helper_1.ErrorCode.RESPONSE_EXCEEDS_MAXIMUM_LENGTH, lastReq));
                }
            });
            res.on('aborted', function handlerStreamAborted() {
                if (rejected) {
                    return;
                }
                res.destroy();
                onError(helper_1.Utils.kyofuucError("Request aborted", config, helper_1.ErrorCode.REQUEST_ABORTED, lastReq));
            });
            res.on('error', function handleStreamError(err) {
                if (lastReq.aborted)
                    return;
                onError(helper_1.Utils.kyofuucError(err, config, helper_1.ErrorCode.RESPONSE_STREAM_ERROR, lastReq));
            });
            res.on('end', function handleStreamEnd() {
                try {
                    let responseData = responseBuffer.length === 1 ? responseBuffer[0] : Buffer.concat(responseBuffer);
                    if (config.responseType !== types_1.ResponseType.ARRAY_BUFFER) {
                        responseData = responseData.toString(config.responseEncoding);
                        if (!config.responseEncoding || config.responseEncoding === 'utf8') {
                            responseData = helper_1.Utils.stripBOM(responseData);
                        }
                    }
                    response.body = responseData;
                    (0, types_1.transformResponseData)(config, response, responseData, onSuccess, onError);
                }
                catch (err) {
                    onError(helper_1.Utils.kyofuucError(err, config, helper_1.ErrorCode.RESPONSE_DATA_PROCESSING_FAILED, response.request, response));
                }
            });
        }
        function transportRequest(redirectCount) {
            var _a;
            const req = transport.request(options, (res) => {
                var _a, _b;
                if (req.aborted || rejected)
                    return;
                const lastReq = (_a = res.req) !== null && _a !== void 0 ? _a : req;
                // handle redirects
                if ((res.statusCode === 301 || res.statusCode === 302) && res.headers['location']) {
                    const newLocation = res.headers['location'];
                    if (config.maxRedirects && (++redirectCount) <= config.maxRedirects) {
                        if (!config.socketPath) {
                            const parsed = helper_1.Utils.parseUrl(newLocation);
                            options.port = parsed.port;
                            options.path = parsed.pathname;
                            options.hostname = parsed.hostname;
                        }
                        if (config.storeRedirectsResponses) {
                            const resolveRedirectResponse = (result) => {
                                redirectsResponses.push(result);
                                req.abort();
                            };
                            processResponse(res, lastReq, resolveRedirectResponse, resolveRedirectResponse);
                        }
                        else {
                            req.abort();
                        }
                        transportRequest(redirectCount);
                        return;
                    }
                    (_b = config.interceptor) === null || _b === void 0 ? void 0 : _b.invoke(core_1.HandlerType.HTTP_REQUEST_MAXIMUM_REDIRECTS_REACHED, config, redirectsResponses);
                }
                // handle decompression
                if (res.statusCode !== 204 && lastReq.method !== 'HEAD' && config.decompress) {
                    try {
                        res = res.pipe(types_1.CompressionProcessor.transform(res.headers['content-encoding'], "decompress"));
                        delete res.headers['content-encoding'];
                    }
                    catch (err) {
                        return reject(helper_1.Utils.kyofuucError(err.message, config, helper_1.ErrorCode.RESPONSE_DECOMPRESSION_ENCODING_INVALID));
                    }
                }
                // handle response
                processResponse(res, lastReq, (result) => {
                    if (config.storeRedirectsResponses) {
                        result.redirectsResponses = redirectsResponses;
                    }
                    helper_1.Utils.resolveResponse(result, resolve, reject);
                }, reject, true);
            });
            req.on('error', (error) => {
                if (req.aborted && error.code !== 'ERR_FR_TOO_MANY_REDIRECTS')
                    return;
                if (error.code && error.code === "ECONNREFUSED" && config.retry) {
                    if (config.retryCount >= config.maxRetry) {
                        reject(helper_1.Utils.kyofuucError(error, config, helper_1.ErrorCode.MAXIMUM_RETRY_EXCEEDED, req));
                        return;
                    }
                    config.retryCount += 1;
                    queueRequest && queueRequest(config);
                    const response = {
                        config,
                        headers: {},
                        status: 150,
                        request: req,
                        __cached__: false,
                        statusText: "Awaiting Retry",
                    };
                    helper_1.Utils.resolveResponse(response, resolve, reject);
                    return;
                }
                ;
                reject(helper_1.Utils.kyofuucError(error, config, helper_1.ErrorCode.REQUEST_ERROR, req));
            });
            req.on('socket', (socket) => {
                if (config.socketKeepAlive) {
                    socket.setKeepAlive(true, config.socketKeepAlive);
                }
            });
            if (config.timeout) {
                req.setTimeout(config.timeout, () => {
                    req.abort();
                    reject(helper_1.Utils.kyofuucError('Timeout of ' + config.timeout + 'ms exceeded', config, helper_1.ErrorCode.REQUEST_TIMEOUT, req));
                });
            }
            if (helper_1.Utils.isStream(data)) {
                if (!(data = (0, types_1.transformRequestData)(config, data, !headerNamesMap['content-type'], headers, reject)))
                    return;
                const stream = data.on('error', (err) => {
                    reject(helper_1.Utils.kyofuucError(err, config, helper_1.ErrorCode.REQUEST_DATA_TRANSFORMATION_FAILED, req));
                }).pipe(req);
                if (config.encoding) {
                    try {
                        headers['Content-Encoding'] = config.encoding;
                        stream.pipe(types_1.CompressionProcessor.transform(config.encoding, "compress"));
                    }
                    catch (err) {
                        return reject(helper_1.Utils.kyofuucError(err.message, config, helper_1.ErrorCode.REQUEST_COMPRESSION_ENCODING_INVALID));
                    }
                }
            }
            else {
                req.end(data);
            }
            if (redirectCount === 0) {
                (_a = config.interceptor) === null || _a === void 0 ? void 0 : _a.invoke(core_1.HandlerType.HTTP_POST_REQUEST, config);
            }
        }
        transportRequest(0);
    });
}
module.exports = httpConnector;
