"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = xhrConnector;
const core_1 = require("../../core");
const node_classes_1 = __importDefault(require("../../helper/node_classes"));
const helper_1 = require("../../helper");
const types_1 = require("../../types");
// TODO treat proxy
// TODO treat cancellation
function xhrConnector(config, queueRequest) {
    return new Promise((resolvePromise, rejectPromise) => {
        var _a, _b;
        let rejected = false;
        function reject(error) {
            rejected = true;
            rejectPromise(error);
        }
        function resolve(response) {
            var _a, _b, _c;
            if (rejected)
                return;
            if (response.__cached__) {
                (_a = config.interceptor) === null || _a === void 0 ? void 0 : _a.invoke(core_1.HandlerType.HTTP_POST_REQUEST, config);
                (_b = config.interceptor) === null || _b === void 0 ? void 0 : _b.invoke(core_1.HandlerType.HTTP_PRE_RESPONSE, config, response);
            }
            (_c = config.interceptor) === null || _c === void 0 ? void 0 : _c.invoke(core_1.HandlerType.HTTP_POST_RESPONSE, config, response);
            resolvePromise(response);
        }
        if (config.dynamicConfig)
            config = config.dynamicConfig(config);
        const preRequestCachedResults = (_a = config.interceptor) === null || _a === void 0 ? void 0 : _a.invoke(core_1.HandlerType.HTTP_PRE_REQUEST, config).filter((r) => (r === null || r === void 0 ? void 0 : r.__cached__) === true);
        if (preRequestCachedResults === null || preRequestCachedResults === void 0 ? void 0 : preRequestCachedResults.length) {
            resolve(preRequestCachedResults[0]);
            return;
        }
        const headers = (_b = config.headers) !== null && _b !== void 0 ? _b : {};
        let data = config.data;
        const headerNamesMap = Object.keys(headers).reduce((acc, header) => {
            acc[header.toLowerCase()] = header;
            return acc;
        }, {});
        if (!helper_1.Utils.envIsBrowser() && !('user-agent' in headerNamesMap)) {
            headers['User-Agent'] = `kyofuuc/${helper_1.Defaults.VERSION}`;
        }
        else {
            if (!headers[headerNamesMap['user-agent']]) {
                delete headers[headerNamesMap['user-agent']];
            }
        }
        if (data && !helper_1.Utils.isStream(data)) {
            if (!config.requestType) {
                if (helper_1.Utils.isString(data)) {
                    config.requestType = types_1.RequestType.TEXT;
                }
                else if (helper_1.Utils.isFormData(data)) {
                    config.requestType = types_1.RequestType.FORM_DATA;
                }
                else if (helper_1.Utils.isURLSearchParams(data)) {
                    config.requestType = types_1.RequestType.URL_SEARCH_PARAMS;
                }
                else if (helper_1.Utils.isPlainObject(data) || helper_1.Utils.isObject(data) || helper_1.Utils.isArray(data)) {
                    config.requestType = types_1.RequestType.JSON;
                }
                else if (helper_1.Utils.isArrayBuffer(data)) {
                    config.requestType = types_1.RequestType.ARRAY_BUFFER;
                }
                else if (Buffer.isBuffer(data)) {
                    config.requestType = types_1.RequestType.BUFFER;
                }
                else {
                    return reject(helper_1.Utils.kyofuucError('Request data after must be one of these ArrayBuffer, string, Stream, Form Data, Json Object, URLSearchParams, or Buffer', config, helper_1.ErrorCode.INVALID_REQUEST_TYPE));
                }
            }
            if (!(data = (0, types_1.transformRequestData)(config, data, !headerNamesMap['content-type'], headers, reject)))
                return;
            if (config.maxContentLength && data.length > config.maxContentLength) {
                return reject(helper_1.Utils.kyofuucError('Request body larger than max content length limit', config, helper_1.ErrorCode.REQUEST_EXCEEDS_MAXIMUM_LENGTH));
            }
            if (headerNamesMap['content-length']) {
                delete headers[headerNamesMap['content-length']];
            }
        }
        if (config.bearer && !headerNamesMap.authorization) {
            headers["Authorization"] = `Bearer ${config.bearer}`;
        }
        if (config.auth && !headerNamesMap.authorization) {
            headers["Authorization"] = `Basic ${helper_1.Utils.toBase64(helper_1.Utils.stringifyAuth(config.auth))}`;
        }
        const redirectsResponses = [];
        function processResponse(res, lastReq, onSuccess, onError, finalRequest = false) {
            var _a, _b, _c;
            const responseLength = helper_1.Utils.length(res.responseData);
            if (config.maxContentLength && responseLength > config.maxContentLength) {
                rejected = true;
                onError(helper_1.Utils.kyofuucError('Max content length of ' + config.maxContentLength + ' exceeded', config, helper_1.ErrorCode.RESPONSE_EXCEEDS_MAXIMUM_LENGTH, lastReq));
                return;
            }
            const response = {
                config,
                request: lastReq,
                __cached__: false,
                headers: res.headers,
                status: res.statusCode,
                statusText: res.statusMessage,
            };
            if (finalRequest) {
                (_a = config.interceptor) === null || _a === void 0 ? void 0 : _a.invoke(core_1.HandlerType.HTTP_PRE_RESPONSE, config, res);
            }
            if (!config.responseType) {
                const contentType = (_c = ((_b = response.headers) !== null && _b !== void 0 ? _b : {})['content-type']) === null || _c === void 0 ? void 0 : _c.toLowerCase();
                if (response.status === 204 || (contentType === null || contentType === void 0 ? void 0 : contentType.includes("nocontent"))) {
                    config.responseType = types_1.ResponseType.NOCONTENT;
                }
                else if (contentType === null || contentType === void 0 ? void 0 : contentType.includes("json")) {
                    config.responseType = types_1.ResponseType.JSON;
                }
                else if (contentType === null || contentType === void 0 ? void 0 : contentType.includes("octet-stream")) {
                    config.responseType = types_1.ResponseType.STREAM;
                }
                else {
                    config.responseType = types_1.ResponseType.TEXT;
                }
            }
            if (config.responseType === types_1.ResponseType.STREAM) {
                response.body = res;
                (0, types_1.transformResponseData)(config, response, res, onSuccess, onError);
                return;
            }
            try {
                let responseData = res.responseData;
                if (config.responseType === types_1.ResponseType.STREAM || config.responseType === types_1.ResponseType.BUFFER || config.responseType === types_1.ResponseType.ARRAY_BUFFER) {
                    responseData = Buffer.from(responseData, 'utf8');
                }
                response.body = responseData;
                (0, types_1.transformResponseData)(config, response, responseData, onSuccess, onError);
            }
            catch (err) {
                onError(helper_1.Utils.kyofuucError(err, config, helper_1.ErrorCode.RESPONSE_DATA_PROCESSING_FAILED, response.request, response));
            }
        }
        function transportRequest(redirectCount) {
            var _a, _b;
            let manuallyAborted = false;
            const req = new node_classes_1.default.XMLHttpRequest();
            req.open(config.method.toUpperCase(), config.url, true);
            function onLoadEnd(onProcess) {
                var _a, _b, _c, _d, _e;
                let responseHeaders = 'getAllResponseHeaders' in req ? helper_1.Utils.parseHeaders(req.getAllResponseHeaders()) : {};
                // handle redirects
                if (!onProcess && (req.status === 301 || req.status === 302) && responseHeaders['location']) {
                    const newLocation = responseHeaders['location'];
                    if (config.maxRedirects && (++redirectCount) <= config.maxRedirects) {
                        manuallyAborted = true;
                        config.url = newLocation;
                        if (config.storeRedirectsResponses) {
                            const resolveRedirectResponse = (result) => {
                                redirectsResponses.push(result);
                                req.abort();
                            };
                            processResponse({
                                statusCode: req.status,
                                headers: responseHeaders,
                                statusMessage: req.statusText,
                                responseData: (_b = (_a = req.response) !== null && _a !== void 0 ? _a : req.responseText) !== null && _b !== void 0 ? _b : req.responseXML,
                            }, req, resolveRedirectResponse, resolveRedirectResponse);
                        }
                        else {
                            req.abort();
                        }
                        transportRequest(redirectCount);
                        return;
                    }
                    (_c = config.interceptor) === null || _c === void 0 ? void 0 : _c.invoke(core_1.HandlerType.HTTP_REQUEST_MAXIMUM_REDIRECTS_REACHED, config, redirectsResponses);
                }
                // handle decompression - TODO
                // handle response
                processResponse({
                    statusCode: req.status,
                    headers: responseHeaders,
                    statusMessage: req.statusText,
                    responseData: (_e = (_d = req.response) !== null && _d !== void 0 ? _d : req.responseText) !== null && _e !== void 0 ? _e : req.responseXML,
                }, req, (result) => {
                    if (onProcess) {
                        onProcess(result);
                        return;
                    }
                    if (config.storeRedirectsResponses) {
                        result.redirectsResponses = redirectsResponses;
                    }
                    helper_1.Utils.resolveResponse(result, resolve, reject);
                }, reject, true);
            }
            if ("onloadend" in req) {
                req.onloadend = () => onLoadEnd();
            }
            else if ("onreadystatechange" in req) {
                req.onreadystatechange = () => {
                    if (!req || req.readyState !== 4) {
                        return;
                    }
                    if (req.status === 0 && !(req.responseURL && req.responseURL.indexOf('file:') === 0)) {
                        return;
                    }
                    setTimeout(onLoadEnd);
                };
            }
            req.onerror = () => {
                if (config.retry) {
                    if (config.retryCount >= config.maxRetry) {
                        onLoadEnd((res) => reject(helper_1.Utils.kyofuucError("Maximum retry exceeded", config, helper_1.ErrorCode.MAXIMUM_RETRY_EXCEEDED, req, res)));
                        return;
                    }
                    config.retryCount += 1;
                    queueRequest && queueRequest(config);
                    const response = {
                        config,
                        headers: {},
                        status: 150,
                        request: req,
                        __cached__: false,
                        statusText: "Awaiting Retry",
                    };
                    helper_1.Utils.resolveResponse(response, resolve, reject);
                    return;
                }
                onLoadEnd((res) => reject(helper_1.Utils.kyofuucError((res.body ? res.body : "Network error"), config, helper_1.ErrorCode.REQUEST_ERROR, req, res)));
            };
            req.onabort = () => {
                if (rejected || manuallyAborted)
                    return;
                reject(helper_1.Utils.kyofuucError("Request aborted", config, helper_1.ErrorCode.REQUEST_ABORTED, req));
            };
            if (config.timeout)
                req.timeout = config.timeout;
            req.ontimeout = () => {
                reject(helper_1.Utils.kyofuucError('Timeout of ' + config.timeout + 'ms exceeded', config, helper_1.ErrorCode.REQUEST_TIMEOUT, req));
            };
            if (config.xsrf) {
                const xsrfValue = (_a = (config.xsrf.cookieName ? helper_1.Utils.getCookie((document !== null && document !== void 0 ? document : {}), config.xsrf.cookieName) : undefined)) !== null && _a !== void 0 ? _a : config.xsrf.value;
                headers[config.xsrf.headerName] = xsrfValue;
            }
            if (config.withCredentials)
                req.withCredentials = config.withCredentials;
            helper_1.Utils.forEach(headers, (key, value) => {
                req.setRequestHeader(key, value);
            });
            if (config.onDownloadProgress)
                req.addEventListener("progress", config.onDownloadProgress);
            if (config.onUploadProgress)
                req.upload.addEventListener("progress", config.onUploadProgress);
            req.send(data);
            if (redirectCount === 0) {
                (_b = config.interceptor) === null || _b === void 0 ? void 0 : _b.invoke(core_1.HandlerType.HTTP_POST_REQUEST, config);
            }
        }
        transportRequest(0);
    });
}
module.exports = xhrConnector;
