"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Defaults = void 0;
const Utils_1 = require("./Utils");
const classes = require("../helper/node_classes");
const types_1 = require("../types");
exports.Defaults = {
    VERSION: "0.0.3",
    MaxObjectEntrySize: 999999,
    MaxCookieLength: 3800, // 3800 Bytes
    MaxStorageSpace: 5120000, // 5000 * 1024 = 5MB
    defaultHttpConnector() {
        let connector;
        if (typeof XMLHttpRequest !== 'undefined') {
            connector = require('../connector/http/xhrConnector');
        }
        else if (typeof process !== 'undefined' && Object.prototype.toString.call(process) === '[object process]') {
            connector = require('../connector/http/httpConnector');
        }
        return connector;
    },
    getDefaultWSConnector() {
        let connector = require('../connector/ws/wsConnector');
        return connector;
    },
    wsConfig(config) {
        if (config.connector === undefined)
            config.connector = exports.Defaults.getDefaultWSConnector();
        return config;
    },
    httpConfig(config) {
        if (config.maxRetry === undefined)
            config.maxRetry = 99999;
        if (config.retryCount === undefined)
            config.retryCount = 0;
        if (config.method === undefined)
            config.method = types_1.Method.GET;
        if (config.maxRedirects === undefined)
            config.maxRedirects = 5;
        if (config.responseEncoding === undefined)
            config.responseEncoding = "utf8";
        if (config.timeout === undefined)
            config.timeout = 1000 * 60 * 5; // 5 minutes
        if (config.connector === undefined)
            config.connector = exports.Defaults.defaultHttpConnector();
        if (Utils_1.Utils.envIsNodeJs() && classes.http) {
            if (!config.httpAgent)
                config.httpAgent = new classes.http.Agent({ keepAlive: true });
            if (!config.httpsAgent)
                config.httpsAgent = new classes.https.Agent({ keepAlive: true });
        }
        if (config.validateStatus === undefined)
            config.validateStatus = (status) => {
                return status >= 100 && status < 300;
            };
        return config;
    },
};
