"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Utils = exports.ErrorCode = void 0;
const buffer_1 = require("buffer");
var ErrorCode;
(function (ErrorCode) {
    ErrorCode["UNKNOWN_ERROR"] = "UNKNOWN_ERROR";
    ErrorCode["REQUEST_ERROR"] = "REQUEST_ERROR";
    ErrorCode["REQUEST_ABORTED"] = "REQUEST_ABORTED";
    ErrorCode["REQUEST_TIMEOUT"] = "REQUEST_TIMEOUT";
    ErrorCode["INVALID_REQUEST_TYPE"] = "INVALID_REQUEST_TYPE";
    ErrorCode["RESPONSE_STREAM_ERROR"] = "RESPONSE_STREAM_ERROR";
    ErrorCode["MAXIMUM_RETRY_EXCEEDED"] = "MAXIMUM_RETRY_EXCEEDED";
    ErrorCode["RESPONSE_STREAM_ABORTED"] = "RESPONSE_STREAM_ABORTED";
    ErrorCode["REQUEST_EXCEEDS_MAXIMUM_LENGTH"] = "REQUEST_EXCEEDS_MAXIMUM_LENGTH";
    ErrorCode["RESPONSE_EXCEEDS_MAXIMUM_LENGTH"] = "RESPONSE_EXCEEDS_MAXIMUM_LENGTH";
    ErrorCode["RESPONSE_DATA_PROCESSING_FAILED"] = "RESPONSE_DATA_PROCESSING_FAILED";
    ErrorCode["REQUEST_DATA_TRANSFORMATION_FAILED"] = "REQUEST_DATA_TRANSFORMATION_FAILED";
    ErrorCode["RESPONSE_DATA_TRANSFORMATION_FAILED"] = "RESPONSE_DATA_TRANSFORMATION_FAILED";
    ErrorCode["REQUEST_COMPRESSION_ENCODING_INVALID"] = "REQUEST_COMPRESSION_ENCODING_INVALID";
    ErrorCode["RESPONSE_STATUS_CODE_FAILED_VALIDATION"] = "RESPONSE_STATUS_CODE_FAILED_VALIDATION";
    ErrorCode["RESPONSE_DECOMPRESSION_ENCODING_INVALID"] = "RESPONSE_DECOMPRESSION_ENCODING_INVALID";
})(ErrorCode || (exports.ErrorCode = ErrorCode = {}));
const _ignoreHeadersDuplicateOf = [
    'age', 'authorization', 'content-length', 'content-type', 'etag',
    'expires', 'from', 'host', 'if-modified-since', 'if-unmodified-since',
    'last-modified', 'location', 'max-forwards', 'proxy-authorization',
    'referer', 'retry-after', 'user-agent'
];
exports.Utils = {
    Buffer: buffer_1.Buffer,
    buildCacheKey(config) {
        var _a;
        if (config.key)
            return config.key;
        const method = (_a = config.method) !== null && _a !== void 0 ? _a : "ANY";
        const suffix = (config.isResource ? "_RESOURCE" : "_SINGLE");
        return `${config.url}_${method}_${suffix}`;
    },
    presentElseImport(item, actual, fallback) {
        if (item !== "undefined")
            return actual.call(actual);
        else
            return fallback.call(fallback);
    },
    safeStringify(obj, indent = 0) {
        let cache = [];
        const retVal = JSON.stringify(obj, (key, value) => typeof value === "object" && value !== null
            ? cache.includes(value)
                ? "[Circular]" // Duplicate reference found, discard key
                : cache.push(value) && value // Store value in our collection
            : value, indent);
        cache = null;
        return retVal;
    },
    // https://stackoverflow.com/a/25812530/6626422
    getCalculateAndSetStorageSize(storage) {
        if (!storage)
            return 0;
        let size = storage.getItem("kyofuuc.calculated.storage.size");
        if (!size) {
            let i = 0;
            try {
                for (i = 250; i <= 10000000000; i += 250) {
                    storage.setItem('kyofuuc.calculated.storage.test', new Array((i * 1024) + 1).join('a'));
                }
                size = "10000000000";
            }
            catch (e) {
                size = `${i - 250}`;
                storage.removeItem('kyofuuc.calculated.storage.test');
                storage.setItem('size', size);
            }
        }
        return parseInt(size);
    },
    getStorageEntries(storage, cb) {
        const items = Object.assign({}, storage);
        for (let key in items) {
            cb(key, items[key]);
        }
    },
    storageSpaceUsed(storage) {
        let used = 0;
        exports.Utils.getStorageEntries(storage, (_, value) => {
            if (`${value}`.endsWith("#_kce_"))
                used += value.length;
        });
        return used;
    },
    getCookieEntries(document, cb) {
        const items = document.cookie.split(/; */);
        for (let item of items) {
            const cookieParts = item.split('=');
            cb(cookieParts[0], cookieParts[1]);
        }
    },
    cookieSpaceUsed(document) {
        let used = 0;
        exports.Utils.getCookieEntries(document, (key, value) => {
            if (`${value}`.endsWith("#_kce_"))
                used += (key.length + value.length);
        });
        return used;
    },
    addCookie(document, options) {
        var _a, _b;
        const path = (_a = options.path) !== null && _a !== void 0 ? _a : "/";
        const expires = options.expires ? `; Expires=${options.expires.toUTCString()}` : "";
        document.cookie = options.name + "=" + ((_b = options.value) !== null && _b !== void 0 ? _b : "") + expires + "; Path=" + path;
    },
    removeCookie(document, name) {
        document.cookie = name + '=; Path=/; Expires=Thu, 01 Jan 1970 00:00:01 GMT;';
    },
    getCookie(document, name) {
        var _a;
        name = name + "=";
        let ca = (_a = document.cookie) === null || _a === void 0 ? void 0 : _a.split(';');
        for (let i = 0; i < ca.length; i++) {
            let c = ca[i];
            while (c.charAt(0) == ' ') {
                c = c.substring(1);
            }
            if (c.indexOf(name) == 0) {
                return c.substring(name.length, c.length);
            }
        }
        return;
    },
    randomString(length = 10, characters = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789") {
        let counter = 0;
        let result = '';
        const charactersLength = characters.length;
        while (counter < length) {
            result += characters.charAt(Math.floor(Math.random() * charactersLength));
            counter += 1;
        }
        return result;
    },
    isArray(val) {
        return Array.isArray(val);
    },
    isURLSearchParams(val) {
        return toString.call(val) === '[object URLSearchParams]';
    },
    isDate(val) {
        return toString.call(val) === '[object Date]';
    },
    isObject(val) {
        return val !== null && typeof val === 'object';
    },
    forEach(obj, fn) {
        if (obj === null || typeof obj === 'undefined') {
            return;
        }
        if (typeof obj !== 'object') {
            obj = [obj];
        }
        if (exports.Utils.isArray(obj)) {
            for (let i = 0, l = obj.length; i < l; i++) {
                fn.call(null, i, obj[i], obj);
            }
        }
        else {
            for (let key in obj) {
                if (Object.prototype.hasOwnProperty.call(obj, key)) {
                    fn.call(null, key, obj[key], obj);
                }
            }
        }
    },
    cherryPick(obj, cherryPicks, remove = false, rebuilds = {}) {
        const picked = Object.keys(obj).reduce((acc, key) => {
            if ((remove && !cherryPicks.includes(key)) || (!remove && cherryPicks.includes(key))) {
                acc[key] = (key in rebuilds ? rebuilds[key] : obj[key]);
            }
            return acc;
        }, {});
        return picked;
    },
    mergeObjects(obj1, obj2, cherryPicks) {
        obj1 = obj1 !== null && obj1 !== void 0 ? obj1 : {};
        obj2 = obj2 !== null && obj2 !== void 0 ? obj2 : {};
        if ((obj1 === null || typeof obj1 === 'undefined') &&
            (obj2 === null || typeof obj2 === 'undefined')) {
            return {};
        }
        let merged = {};
        if (typeof obj1 !== 'object' && typeof obj2 === 'object') {
            merged = obj2;
            return merged;
        }
        if (typeof obj2 !== 'object') {
            merged = obj1;
            return merged;
        }
        if (cherryPicks) {
            for (const cherryPick of cherryPicks) {
                if (cherryPick in obj2)
                    merged[cherryPick] = obj2[cherryPick];
                else if (cherryPick in obj1)
                    merged[cherryPick] = obj1[cherryPick];
            }
            return merged;
        }
        for (let key in obj2) {
            merged[key] = obj2[key];
        }
        for (let key in obj1) {
            if (merged[key] === undefined || merged[key] === null) {
                merged[key] = obj1[key];
            }
        }
        return merged;
    },
    encodeParamURI(val) {
        return encodeURIComponent(val).
            replace(/%5B/gi, '[').
            replace(/%5D/gi, ']').
            replace(/%3A/gi, ':').
            replace(/%24/g, '$').
            replace(/%20/g, '+').
            replace(/%2C/gi, ',');
    },
    isAbsoluteURL(url) {
        return /^([a-z][a-z\d+\-.]*:)?\/\//i.test(url);
    },
    isRelativeURL(url) {
        return !exports.Utils.isAbsoluteURL(url);
    },
    combineUrls(baseUrl, relativeUrl) {
        if (!baseUrl)
            return relativeUrl;
        return relativeUrl
            ? baseUrl.replace(/\/+$/, '') + '/' + relativeUrl.replace(/^\/+/, '')
            : baseUrl;
    },
    buildFullUrl(config, stripAuth) {
        var _a;
        let fullUrl = (_a = config.url) !== null && _a !== void 0 ? _a : config.baseUrl;
        if (config.baseUrl && config.url && !exports.Utils.isAbsoluteURL(config.url)) {
            fullUrl = exports.Utils.combineUrls(config.baseUrl, config.url);
        }
        if (!fullUrl || !stripAuth)
            return fullUrl;
        return exports.Utils.stripAuthFromUrl(fullUrl);
    },
    stripAuthFromUrl(fullUrl) {
        let zeroAtIndex = fullUrl.indexOf("@");
        const protocolIndex = fullUrl.indexOf("://");
        const authSeparatorIndex = fullUrl.indexOf(":", (protocolIndex > -1 ? (protocolIndex + 3) : 0));
        let atIndex = fullUrl.indexOf("@", (authSeparatorIndex > -1 ? (authSeparatorIndex + 1) : 0));
        if (atIndex === -1)
            atIndex = zeroAtIndex;
        if (atIndex === -1)
            return fullUrl;
        return fullUrl.substring(0, protocolIndex + (protocolIndex > -1 ? 3 : 0)) + fullUrl.substring(atIndex + 1);
    },
    buildUrlWithQuery(url, query, querySerializer) {
        let serializedQuery;
        if (exports.Utils.isURLSearchParams(query)) {
            serializedQuery = query.toString();
        }
        else if (querySerializer) {
            serializedQuery = querySerializer(query);
        }
        else {
            let queryParts = [];
            exports.Utils.forEach(query, function serialize(key, value) {
                if (value === null || typeof value === 'undefined') {
                    return;
                }
                if (exports.Utils.isArray(value)) {
                    key = key + '[]';
                }
                else {
                    value = [value];
                }
                exports.Utils.forEach(value, function parseValue(_, val) {
                    if (exports.Utils.isDate(val)) {
                        val = val.toISOString();
                    }
                    else if (exports.Utils.isObject(val)) {
                        val = exports.Utils.safeStringify(val);
                    }
                    queryParts.push(`${exports.Utils.encodeParamURI(key)}=${exports.Utils.encodeParamURI(val)}`);
                });
                serializedQuery = queryParts.join('&');
            });
        }
        if (serializedQuery) {
            var hashIndex = url.indexOf('#');
            if (hashIndex !== -1) {
                url = url.slice(0, hashIndex);
            }
            url += (url.indexOf('?') === -1 ? '?' : '&') + serializedQuery;
        }
        return url;
    },
    buildUrlWithQueryFromConfig(config) {
        var _a;
        if (!config.url)
            return undefined;
        return exports.Utils.buildUrlWithQuery(config.url, (_a = config.query) !== null && _a !== void 0 ? _a : {}, config.querySerializer);
    },
    basicAuthFromUrl(fullUrl) {
        var _a;
        let atIndex = fullUrl.indexOf("@");
        if (atIndex === -1)
            return undefined;
        let doubleSlashIndex = (_a = fullUrl.indexOf("//")) !== null && _a !== void 0 ? _a : -1;
        let hasDoubleSlashIndex = doubleSlashIndex > -1;
        const authParts = (fullUrl === null || fullUrl === void 0 ? void 0 : fullUrl.substring(doubleSlashIndex + (hasDoubleSlashIndex ? 2 : 0), fullUrl === null || fullUrl === void 0 ? void 0 : fullUrl.indexOf("@"))).split(":");
        return {
            username: encodeURIComponent(authParts.length > 0 ? authParts[0] : ""),
            password: encodeURIComponent(authParts.length > 1 ? authParts[1] : ""),
        };
    },
    parseUrl(url) {
        return new URL(url);
    },
    isFunction(val) {
        return toString.call(val) === '[object Function]';
    },
    isStream(val) {
        return exports.Utils.isObject(val) && exports.Utils.isFunction(val.pipe);
    },
    isString(val) {
        return typeof val === 'string';
    },
    isPlainObject(val) {
        if (toString.call(val) !== '[object Object]') {
            return false;
        }
        let prototype = Object.getPrototypeOf(val);
        return prototype === null || prototype === Object.prototype;
    },
    isArrayBuffer(val) {
        return toString.call(val) === '[object ArrayBuffer]';
    },
    isFormData(val) {
        return toString.call(val) === '[object FormData]';
    },
    kyofuucError(message, config, code, request, response) {
        let error = (typeof message === "string" ? new Error(message) : message);
        error.config = config;
        if (code) {
            error.code = code;
        }
        error.request = request;
        error.response = response;
        error.isKyofuucError = true;
        error.toJSON = function toJSON() {
            var _a, _b;
            return {
                name: this.name,
                code: this.code,
                stack: this.stack,
                config: this.config,
                number: this.number,
                message: this.message,
                fileName: this.fileName,
                lineNumber: this.lineNumber,
                description: this.description,
                status: (_a = this.response) === null || _a === void 0 ? void 0 : _a.status,
                columnNumber: this.columnNumber,
                responseBody: (_b = this.response) === null || _b === void 0 ? void 0 : _b.body,
            };
        };
        return error;
    },
    envIsNodeJs() {
        return (typeof process !== 'undefined' && Object.prototype.toString.call(process) === '[object process]');
    },
    resolveResponse(response, resolve, reject) {
        var _a;
        let validateStatus = (_a = response.config) === null || _a === void 0 ? void 0 : _a.validateStatus;
        if (!response.status || !validateStatus || validateStatus(response.status)) {
            resolve(response);
        }
        else {
            reject(exports.Utils.kyofuucError('Request failed with status code ' + response.status, response.config, ErrorCode.RESPONSE_STATUS_CODE_FAILED_VALIDATION, response.request, response));
        }
    },
    toArrayBuffer(buffer) {
        const arrayBuffer = new ArrayBuffer(buffer.length);
        const view = new Uint8Array(arrayBuffer);
        for (let i = 0; i < buffer.length; ++i) {
            view[i] = buffer[i];
        }
        return arrayBuffer;
    },
    stripBOM(content) {
        if (content.charCodeAt(0) === 0xFEFF) {
            content = content.slice(1);
        }
        return content;
    },
    stringifyAuth(auth) {
        if (!auth)
            return undefined;
        return `${decodeURIComponent(auth.username)}:${decodeURIComponent(auth.password)}`;
    },
    envIsBrowser() {
        return !(typeof window === 'undefined');
    },
    toBase64(value) {
        if (exports.Utils.envIsBrowser()) {
            return btoa(value);
        }
        return buffer_1.Buffer.from(value).toString('base64');
    },
    fromBase64(value) {
        if (exports.Utils.envIsBrowser()) {
            return atob(value);
        }
        return buffer_1.Buffer.from(value, 'base64').toString();
    },
    trim(str) {
        return str.trim ? str.trim() : str.replace(/^\s+|\s+$/g, '');
    },
    length(str) {
        var _a;
        return (_a = str.length) !== null && _a !== void 0 ? _a : 0;
    },
    parseHeaders(headers) {
        let i;
        let key;
        let val;
        let parsed = {};
        if (!headers) {
            return parsed;
        }
        exports.Utils.forEach(headers.split('\n'), (_, line) => {
            i = line.indexOf(':');
            key = exports.Utils.trim(line.substring(0, i)).toLowerCase();
            val = exports.Utils.trim(line.substring(i + 1));
            if (key) {
                if (parsed[key] && _ignoreHeadersDuplicateOf.indexOf(key) >= 0) {
                    return;
                }
                if (key === 'set-cookie') {
                    parsed[key] = (parsed[key] ? parsed[key] : []).concat([val]);
                }
                else {
                    parsed[key] = parsed[key] ? parsed[key] + ', ' + val : val;
                }
            }
        });
        return parsed;
    },
    addMsToDate(date, ms) {
        date.setMilliseconds(date.getMilliseconds() + ms);
        return date;
    },
    dateDiffInMs(date1, date2) {
        return date1.getTime() - date2.getTime();
    },
};
