"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __exportStar = (this && this.__exportStar) || function(m, exports) {
    for (var p in m) if (p !== "default" && !Object.prototype.hasOwnProperty.call(exports, p)) __createBinding(exports, m, p);
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ffs = exports.Ffs = void 0;
const node_classes_1 = __importDefault(require("./helper/node_classes"));
const xhrConnector_1 = __importDefault(require("./connector/http/xhrConnector"));
const httpConnector_1 = __importDefault(require("./connector/http/httpConnector"));
const helper_1 = require("./helper");
const types_1 = require("./types");
const core_1 = require("./core");
const cachemanager_1 = require("./cachemanager");
const exception_1 = require("./exception");
class Ffs {
    constructor(config) {
        this.Ws = core_1.Ws;
        this.Ffs = Ffs;
        this.Http = core_1.Http;
        this.Utils = helper_1.Utils;
        this.Method = types_1.Method;
        this.WsState = core_1.WsState;
        this.Defaults = helper_1.Defaults;
        this.http = node_classes_1.default.http;
        this.https = node_classes_1.default.https;
        this.EventQueue = core_1.EventQueue;
        this.RequestType = types_1.RequestType;
        this.Interceptor = core_1.Interceptor;
        this.HandlerType = core_1.HandlerType;
        this.xhrConnector = xhrConnector_1.default;
        this.ResponseType = types_1.ResponseType;
        this.httpConnector = httpConnector_1.default;
        this.WebSocket = node_classes_1.default.WebSocket;
        this.EventQueueType = core_1.EventQueueType;
        this.MapCacheManager = cachemanager_1.MapCacheManager;
        this.CompressionType = types_1.CompressionType;
        this.RequestProcessor = types_1.RequestProcessor;
        this.ResponseProcessor = types_1.ResponseProcessor;
        this.MissingCacheError = exception_1.MissingCacheError;
        this.XMLHttpRequest = node_classes_1.default.XMLHttpRequest;
        this.CookieCacheManager = cachemanager_1.CookieCacheManager;
        this.StorageCacheManager = cachemanager_1.StorageCacheManager;
        this.CompressionProcessor = types_1.CompressionProcessor;
        this.transformRequestData = types_1.transformRequestData;
        this.transformResponseData = types_1.transformResponseData;
        this.InvalidParameterError = exception_1.InvalidParameterError;
        this.NoEventFoundWithIdError = exception_1.NoEventFoundWithIdError;
        this.LocalStorageCacheManager = cachemanager_1.LocalStorageCacheManager;
        this.SessionStorageCacheManager = cachemanager_1.SessionStorageCacheManager;
        this.UnregisteredRequestTypeError = exception_1.UnregisteredRequestTypeError;
        this.UnregisteredResponseTypeError = exception_1.UnregisteredResponseTypeError;
        this.NoSufficientCacheSpaceLeftError = exception_1.NoSufficientCacheSpaceLeftError;
        this._ws = new core_1.Ws(config);
        this._http = new core_1.Http(config);
    }
    static init(config) {
        return new Ffs(config);
    }
    init(config) {
        return Ffs.init(config);
    }
    // http
    getConfig(type) {
        if (type === "WS")
            return this._ws.getConfig();
        return this._http.getConfig();
    }
    updateConfig(newConfig, merge) {
        this._http.updateConfig(newConfig, merge);
    }
    getUrl(config) {
        return this._http.getUrl(config);
    }
    queueRequest(config) {
        return this._http.queueRequest(config);
    }
    async retryRequests(cache) {
        return this._http.retryRequests(cache);
    }
    get(url, config) {
        return this._http.get(url, config);
    }
    head(url, config) {
        return this._http.head(url, config);
    }
    delete(url, config) {
        return this._http.delete(url, config);
    }
    options(url, config) {
        return this._http.options(url, config);
    }
    put(url, data, config) {
        return this._http.put(url, data, config);
    }
    post(url, data, config) {
        return this._http.post(url, data, config);
    }
    patch(url, data, config) {
        return this._http.patch(url, data, config);
    }
    request(urlOrConfig, config) {
        return this._http.request(urlOrConfig, config);
    }
    // ws
    ws(config) {
        return core_1.Ws.ws(config);
    }
    close() {
        return this._ws.close();
    }
    getState() {
        return this._ws.getState();
    }
    getBufferedAmount() {
        return this._ws.getBufferedAmount();
    }
    sendMessage(message) {
        this._ws.sendMessage(message);
    }
    getLastReconnectionCount() {
        return this._ws.getLastReconnectionCount();
    }
    getBinaryType() {
        return this._ws.getBinaryType();
    }
    setBinaryType(binaryType) {
        this._ws.setBinaryType(binaryType);
    }
    reconnect(connector) {
        return this._ws.reconnect(connector);
    }
    getConnection() {
        return this._ws.getConnection();
    }
    connect(urlOrConfig, config) {
        return this._ws.connect(urlOrConfig, config);
    }
    onOpen(cb, options) {
        this._ws.onOpen(cb, options);
    }
    onClose(cb, options, always) {
        this._ws.onClose(cb, options, always);
    }
    onError(cb, options) {
        this._ws.onError(cb, options);
    }
    onStateChange(cb, options) {
        this._ws.onStateChange(cb, options);
    }
    onMessage(cb, options) {
        this._ws.onMessage(cb, options);
    }
}
exports.Ffs = Ffs;
__exportStar(require("./core"), exports);
__exportStar(require("./types"), exports);
__exportStar(require("./helper"), exports);
__exportStar(require("./exception"), exports);
__exportStar(require("./cachemanager"), exports);
__exportStar(require("./connector/http/xhrConnector"), exports);
__exportStar(require("./connector/http/httpConnector"), exports);
exports.ffs = new Ffs();
exports.default = exports.ffs;
module.exports = exports.ffs;
module.exports.default = exports.ffs;
